@@ -84,6 +84,7 @@ static struct l7vs_protomod cinsert_protomod = {
 	NULL,           /* handle */
 	"cinsert",      /* modname */
 	0,              /* refcnt */
+	0,              /* fast schedule */
 	create,         /* create function */
 	compare,        /* compare function */
 	match_cldata,   /* match_cldata function */
@@ -749,15 +750,24 @@ analyze_rsdata(struct l7vs_service* srv, struct l7vs_conn* conn,
 	/* create encode strings of IP address and port number */
 	http_encode_address(encoded_address, conn->dest->addr.sin_addr.s_addr, conn->dest->addr.sin_port);
 
-	/* calculate expire time */
-	expire_time = l7vs_protomod_cinsert_timeout(cinsert_service->cookie_expire);
-
-	/* create expire strings */
-	http_cookie_expire(expire_time, expire_strings);
-
-	/* create Set-Cookie header field */
-	snprintf(set_cookie_field, L7VS_PROTOMOD_MAX_ADD_BUFSIZE, "Set-Cookie: %s=%s; expires=%s; path=/\r\n",
-	        cinsert_service->cookie_name, encoded_address, expire_strings);
+	/* expire cookie */
+	if (cinsert_service->cookie_expire) {
+		/* calculate expire time */
+		expire_time = l7vs_protomod_cinsert_timeout(cinsert_service->cookie_expire);
+	
+		/* create expire strings */
+		http_cookie_expire(expire_time, expire_strings);
+	
+		/* create Set-Cookie header field */
+		snprintf(set_cookie_field, L7VS_PROTOMOD_MAX_ADD_BUFSIZE, "Set-Cookie: %s=%s; expires=%s; path=/\r\n",
+		        cinsert_service->cookie_name, encoded_address, expire_strings);
+	}
+	/* temporary cookie */
+	else {
+		/* create Set-Cookie header field */
+		snprintf(set_cookie_field, L7VS_PROTOMOD_MAX_ADD_BUFSIZE, "Set-Cookie: %s=%s; path=/\r\n",
+		        cinsert_service->cookie_name, encoded_address);
+	}
 
 	/*-------- DEBUG LOG --------*/
 	if (cinsert_protomod.get_log_level != NULL &&
