@@ -5,6 +5,7 @@
  *
  * L7VSD: Linux Virtual Server for Layer7 Load Balancing
  * Copyright (C) 2008  NTT COMWARE Corporation.
+ * Copyright (C) 2009  Shinya TAKEBAYASHI
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,39 +38,8 @@
 #include "l7vs_replication.h"
 #include "l7vs_module.h"
 #include "module_http.h"
-
-#define SERVICE_ARG_MAXSIZE  (512)
-#define SSLID_SERVICE_NUMBER (128)
-#define SSLID_LENGTH         (32)
-
-struct l7vs_sslid_service {
-	handle_t service_handle;
-	int timeout;
-	int maxlist;
-	struct ssl_session* session;
-	void* replication_addr;
-	int reschedule;
-};
-
-struct l7vs_sslid_service_arg {
-	int timeout;
-	int maxlist;
-	int reschedule;
-};
-
-struct ssl_session {
-	char id[SSLID_LENGTH];
-	struct l7vs_dest dest;
-	time_t last_time;
-	char valid;
-};
-
-struct replication_header {
-	struct in_addr sin_addr;
-	unsigned short sin_port;
-	unsigned int size;
-	unsigned int offset;
-};
+#include "module_sslid_hash.h"
+#include "protomod_sslid.h"
 
 static void  fini(void);
 static int   create(void*, handle_t);
@@ -103,6 +73,7 @@ static struct l7vs_protomod sslid_protomod = {
 	NULL,           /* handle */
 	"sslid",        /* modname */
 	0,              /* refcnt */
+        0,              /* fast schedule */
 	create,         /* create function */
 	compare,        /* compare function */
 	match_cldata,   /* match_cldata function */
@@ -203,6 +174,8 @@ fini(void)
 
 				free(sslid_service_list[service_number]->session);
 				sslid_service_list[service_number]->session = NULL;
+				hash_setPointer(sslid_service_list[service_number]->hash_map, sslid_service_list[service_number]->hash_list, sslid_service_list[service_number]->maxlist);
+				hash_destroy();
 			}
 
 			/*-------- DEBUG LOG --------*/
@@ -297,6 +270,9 @@ create(void* sslid_arg, handle_t service_handle)
 	if (service_handle != TEMP_SERVICEHANDLE) {
 		/* create session area */
 		sslid_service->session = (struct ssl_session*) calloc(sizeof(struct ssl_session), sslid_service->maxlist);
+		hash_allocate(sslid_service_arg->maxlist);
+		sslid_service->hash_map = hash_getIDMAP();
+		sslid_service->hash_list = hash_getIDLIST();
 
 		/*-------- DEBUG LOG --------*/
 		if (sslid_protomod.get_log_level != NULL &&
@@ -446,6 +422,9 @@ match_cldata(struct l7vs_service* srv, struct l7vs_conn* conn,
 	int ret;
 	int return_value = 0;
 	time_t now;
+	char id_str[DEBUG_STR_LEN] = {0};
+	struct l7vs_dest *tmpdest = NULL;
+	int searchret = -1;
 
 	/*-------- DEBUG LOG --------*/
 	if (sslid_protomod.get_log_level != NULL &&
@@ -535,6 +514,7 @@ match_cldata(struct l7vs_service* srv, struct l7vs_conn* conn,
 
 	/* read replication data */
 	l7vs_protomod_sslid_read_replication_data(sslid_service, srv);
+	hash_rebuild_sessionlist(sslid_service);
 
 	/* initialize protocol module ... clear destination list */
 	ret = srv->pm->initialize(srv, conn, request, *len, dest);
@@ -564,30 +544,14 @@ match_cldata(struct l7vs_service* srv, struct l7vs_conn* conn,
 		}
 		/*------ DEBUG LOG END ------*/
 
-		time(&now);
-		for (i = 0; i < sslid_service->maxlist; i++) {
-			if ((sslid_service->session + i)->valid == 1 && 
-			    memcmp((sslid_service->session + i)->id, &request[44], 32) == 0) {
-				if (sslid_service->timeout > 0 &&
-				    now - (sslid_service->session + i)->last_time > sslid_service->timeout) {
-					(sslid_service->session + i)->valid = 0;
-				}
-				else {
-					*dest = &(sslid_service->session + i)->dest;
-
-					/*-------- DEBUG LOG --------*/
-					if (sslid_protomod.get_log_level != NULL &&
-					    LOG_LV_DEBUG == sslid_protomod.get_log_level(LOG_CAT_L7VSD_PROTOCOL)) {
-						char dest_str[DEBUG_STR_LEN] = {0};
-						l7vs_dest_c_str(dest_str, *dest);
-						PUT_LOG_DEBUG(sslid_protomod, LOG_CAT_L7VSD_PROTOCOL,106, "pointer assign: dest=&(&(%s))",
-						    dest_str);
-					}
-					/*------ DEBUG LOG END ------*/
-				}
-				break;
-			}
+		hash_setPointer(sslid_service->hash_map, sslid_service->hash_list, sslid_service->maxlist);
+		id_c_str(id_str, &request[44]);
+		searchret = hash_search(id_str, &tmpdest);
+		
+		if (searchret == 0) {
+			*dest = tmpdest;
 		}
+		
 	}
 
 	*tcps = 0;
@@ -634,6 +598,7 @@ analyze_rsdata(struct l7vs_service* srv, struct l7vs_conn* conn,
 	int pick;
 	time_t now, oldest = 0;
 	char session_full = 1;
+	char id_str[DEBUG_STR_LEN] = {0};
 
 	/*-------- DEBUG LOG --------*/
 	if (sslid_protomod.get_log_level != NULL &&
@@ -741,61 +706,11 @@ analyze_rsdata(struct l7vs_service* srv, struct l7vs_conn* conn,
 		}
 		/*------ DEBUG LOG END ------*/
 
-		time(&now);
-		for (i = 0; i < sslid_service->maxlist; i++) {
-			if ((sslid_service->session + i)->valid == 1 &&
-			    memcmp((sslid_service->session + i)->id, &response[44], 32) == 0) {
-				exist = 1;
-				(sslid_service->session + i)->last_time = now;
-				break;
-			}
-		}
-		if (!exist) {
-			for (i = 0; i < sslid_service->maxlist; i++) {
-				if ((sslid_service->session + i)->valid == 0) {
-					pick = i;
-					session_full = 0;
-					break;
-				}
-				if (oldest == 0 || oldest > (sslid_service->session + i)->last_time) {
-					pick = i;
-					oldest = (sslid_service->session + i)->last_time;
-				}
-			}
-			if (session_full) {
-				PUT_LOG_INFO(sslid_protomod, LOG_CAT_L7VSD_PROTOCOL,5,
-				    "Overwrite oldest session 'cause all session is valid.");
-			}
-			(sslid_service->session + pick)->valid = 1;
-			(sslid_service->session + pick)->last_time = now;
-			memcpy((sslid_service->session + pick)->id, &response[44], 32);
-
-			/*-------- DEBUG LOG --------*/
-			if (sslid_protomod.get_log_level != NULL &&
-			    LOG_LV_DEBUG == sslid_protomod.get_log_level(LOG_CAT_L7VSD_PROTOCOL)) {
-				char id_str[DEBUG_STR_LEN] = {0};
-				id_c_str(id_str, (sslid_service->session + pick)->id);
-				PUT_LOG_DEBUG(sslid_protomod, LOG_CAT_L7VSD_PROTOCOL,112,
-				    "pointer assign: (sslid_service->session + %d)->id=\"%s\"",
-				    i, id_str);
-			}
-			/*------ DEBUG LOG END ------*/
-
-			memcpy(&(sslid_service->session + pick)->dest, conn->dest, sizeof(struct l7vs_dest));
-
-			/*-------- DEBUG LOG --------*/
-			if (sslid_protomod.get_log_level != NULL &&
-			    LOG_LV_DEBUG == sslid_protomod.get_log_level(LOG_CAT_L7VSD_PROTOCOL)) {
-				char dest_str[DEBUG_STR_LEN] = {0};
-				l7vs_dest_c_str(dest_str, &(sslid_service->session + pick)->dest);
-				PUT_LOG_DEBUG(sslid_protomod, LOG_CAT_L7VSD_PROTOCOL,113,
-				    "pointer assign: &(sslid_service->session + %d)->dest=&(%s)",
-				    i, dest_str);
-			}
-			/*------ DEBUG LOG END ------*/
-
-			l7vs_protomod_sslid_write_replication_data(sslid_service);
-		}
+		hash_setPointer(sslid_service->hash_map, sslid_service->hash_list, sslid_service->maxlist);
+		id_c_str(id_str, &response[44]);
+		hash_add(id_str, *conn->dest);
+		hash_construct_sessionlist(sslid_service);
+		l7vs_protomod_sslid_write_replication_data(sslid_service);
 	}
 
 analyze_rsdata_out:
@@ -853,6 +768,8 @@ destroy(handle_t srv_handle)
 
 				free(sslid_service_list[service_number]->session);
 				sslid_service_list[service_number]->session = NULL;
+				hash_setPointer(sslid_service_list[service_number]->hash_map, sslid_service_list[service_number]->hash_list, sslid_service_list[service_number]->maxlist);
+				hash_destroy();
 			}
 
 			/*-------- DEBUG LOG --------*/
