@@ -36,21 +36,39 @@
 #include "module_http.h"
 #include <fnmatch.h>
 
+#include <iostream>
+#ifdef USE_BOOST_XPRESSIVE
+    #include <boost/xpressive/xpressive.hpp>
+#else
+    #include <boost/regex.hpp>
+#endif
 
-#define SERVICE_ARG_MAXSIZE    (512)
-#define PATTERN_MATCH_MAXSIZE  (128)
-#define URL_SERVICE_NUMBER     (128)
-#define X_FORWARDED_FOR_LENGTH (48)
+#define SERVICE_ARG_MAXSIZE          (512)
+#define PATTERN_MATCH_MAXSIZE        (128)
+#define REGEX_PATTERN_MATCH_MAXSIZE  (32)
+#define URL_SERVICE_NUMBER           (128)
+#define X_FORWARDED_FOR_LENGTH       (48)
 
 struct l7vs_url_service {
 	handle_t service_handle;
 	char pattern_match[PATTERN_MATCH_MAXSIZE];
+	char uri_pattern_match[REGEX_PATTERN_MATCH_MAXSIZE];
+	char host_pattern_match[REGEX_PATTERN_MATCH_MAXSIZE];
 	int forwarded_for;
+#ifdef USE_BOOST_XPRESSIVE
+	boost::xpressive::sregex uri_regex;
+	boost::xpressive::sregex host_regex;
+#else
+	boost::regex uri_regex;
+	boost::regex host_regex;
+#endif
 	int reschedule;
 };
 
 struct  l7vs_url_service_arg {
 	char pattern_match[PATTERN_MATCH_MAXSIZE];
+	char uri_pattern_match[REGEX_PATTERN_MATCH_MAXSIZE];
+	char host_pattern_match[REGEX_PATTERN_MATCH_MAXSIZE];
 	int forwarded_for;
 	int reschedule;
 };
@@ -81,6 +99,7 @@ static struct l7vs_protomod url_protomod = {
 	NULL,           /* handle */
 	"url",          /* modname */
 	0,              /* refcnt */
+	0,              /* fast schedule */
 	create,         /* create function */
 	compare,        /* compare function */
 	match_cldata,   /* match_cldata function */
@@ -250,7 +269,37 @@ create(void *url_arg, handle_t service_handle)
 	/* set service handle, pattern match and reschedule flag */
 	url_service->service_handle = service_handle;
 	strncpy(url_service->pattern_match, url_service_arg->pattern_match, PATTERN_MATCH_MAXSIZE);
+	strncpy(url_service->uri_pattern_match, url_service_arg->uri_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE);
+	strncpy(url_service->host_pattern_match, url_service_arg->host_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE);
 	url_service->forwarded_for = url_service_arg->forwarded_for;
+	if ( strnlen(url_service_arg->uri_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE) ) {
+		try {
+#ifdef USE_BOOST_XPRESSIVE
+			url_service->uri_regex = boost::xpressive::sregex::compile(url_service_arg->uri_pattern_match);
+#else
+			url_service->uri_regex.assign(url_service_arg->uri_pattern_match);
+#endif
+		}
+		catch (...) {
+			PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,284, "Regex pattern error of uri_pattern_match");
+			return_value = -1;
+			goto create_out;
+		}
+	}
+	if ( strnlen(url_service_arg->host_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE) ) {
+		try {
+#ifdef USE_BOOST_XPRESSIVE
+			url_service->host_regex = boost::xpressive::sregex::compile(url_service_arg->host_pattern_match);
+#else
+			url_service->host_regex.assign(url_service_arg->host_pattern_match);
+#endif
+		}
+		catch (...) {
+			PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,285, "Regex pattern error of host_pattern_match.");
+			return_value = -1;
+			goto create_out;
+		}
+	}
 	url_service->reschedule = url_service_arg->reschedule;
 
 create_out:
@@ -392,7 +441,15 @@ compare(handle_t srv_handle1, handle_t srv_handle2)
 	}
 
 	/* compare two pattern match */
-	if (strcmp(url_srv1->pattern_match, url_srv2->pattern_match) != 0) {
+	if (strncmp(url_srv1->pattern_match, url_srv2->pattern_match, PATTERN_MATCH_MAXSIZE) != 0) {
+		return_value = -1;
+		goto compare_out;
+	}
+	if (strncmp(url_srv1->uri_pattern_match, url_srv2->uri_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE) != 0) {
+		return_value = -1;
+		goto compare_out;
+	}
+	if (strncmp(url_srv1->host_pattern_match, url_srv2->host_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE) != 0) {
 		return_value = -1;
 		goto compare_out;
 	}
@@ -431,7 +488,7 @@ match_cldata(struct l7vs_service *srv, struct l7vs_conn *conn,
 	int asterisk_num = 0;
 	size_t pattern_len;
 	size_t uri_len;
-	char *uri,*host,pattern[PATTERN_MATCH_MAXSIZE+2];
+	char *uri, *host, pattern[PATTERN_MATCH_MAXSIZE + 2];
 	int offset_length;
 	char *x_forwarded_value;
 	char *next_line = NULL;
@@ -439,6 +496,8 @@ match_cldata(struct l7vs_service *srv, struct l7vs_conn *conn,
 	char backup_char;
 	int return_value = 0;
 
+	std::string packet_data;
+
 	/*-------- DEBUG LOG --------*/
 	if (url_protomod.get_log_level != NULL &&
 	    LOG_LV_DEBUG == url_protomod.get_log_level(LOG_CAT_L7VSD_PROTOCOL)) {
@@ -534,28 +593,26 @@ match_cldata(struct l7vs_service *srv, struct l7vs_conn *conn,
 		goto match_cldata_out;
 	}
 
-	/* check pattern_match != 0 */
-	if (url_service->pattern_match[0] == '\0') {
-		PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,131, "Pattern match is NULL pointer");
-		return_value = -1;
-		goto match_cldata_out;
-	}
-
-	/* check the size of request data */
-	pattern_len = strnlen(url_service->pattern_match, PATTERN_MATCH_MAXSIZE);
-	/* When '*' is included in pattern_match, number of '*' is decreased from pattern_len */ 
-	for (i = 0; i < (int)pattern_len; i++) {
-		if (url_service->pattern_match[i] == '*') {
-			asterisk_num++;
+	/* check pattern_match length */
+	if (url_service->pattern_match[0] != '\0') {
+// remove log because of add new key --uri-pattern-match, --host-pattern-match)
+//		PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,131, "Pattern match is NULL pointer");
+		/* check the size of request data */
+		pattern_len = strnlen(url_service->pattern_match, PATTERN_MATCH_MAXSIZE);
+		/* When '*' is included in pattern_match, number of '*' is decreased from pattern_len */ 
+		for (i = 0; i < (int)pattern_len; i++) {
+			if (url_service->pattern_match[i] == '*') {
+				asterisk_num++;
+			}
+		}
+		pattern_len -= (size_t)asterisk_num;
+		if (*len < (15 + pattern_len) ) {
+			PUT_LOG_INFO(url_protomod, LOG_CAT_L7VSD_PROTOCOL,6, "Request data is too short.");
+			return_value = 1;
+			goto match_cldata_out;
 		}
 	}
-	pattern_len -= (size_t)asterisk_num;
-	if (*len < (15 + pattern_len) ) {
-		PUT_LOG_INFO(url_protomod, LOG_CAT_L7VSD_PROTOCOL,6, "Request data is too short.");
-		return_value = 1;
-		goto match_cldata_out;
-	}
-
+	
 	/* check the HTTP method in HTTP request header */
 	uri_len = *len;
 	uri = http_check_request_method(request, &uri_len);
@@ -565,13 +622,28 @@ match_cldata(struct l7vs_service *srv, struct l7vs_conn *conn,
 		goto match_cldata_out;
 	}
 	
-	/* check keyword (URI)*/
-	snprintf(pattern, pattern_len + 3, "*%s*", url_service->pattern_match);
-	backup_char = uri[uri_len];
-	uri[uri_len] = '\0';
-	ret = fnmatch(pattern, uri, 0);
-	uri[uri_len] = backup_char;
-	if (ret != 0) {
+	/* check keyword (URI/--uri-pattern-match)*/
+	if (url_service->uri_pattern_match[0] != '\0') {
+		packet_data = std::string(uri, uri_len);
+#ifdef USE_BOOST_XPRESSIVE
+		if ( !boost::xpressive::regex_search(packet_data, url_service->uri_regex) ) {
+#else
+		if ( !boost::regex_search(packet_data, url_service->uri_regex) ) {
+#endif
+			return_value = -1;
+			goto match_cldata_out;
+		}
+	}
+	/* check keyword (URI/--pattern-match)*/
+	if (url_service->pattern_match[0] != '\0') {
+		backup_char = uri[uri_len];
+		uri[uri_len] = '\0';
+		snprintf(pattern, pattern_len + 3, "*%s*", url_service->pattern_match);
+		ret = fnmatch(pattern, uri, 0);
+		uri[uri_len] = backup_char;
+	}
+	if ( (url_service->pattern_match[0] != '\0' && ret != 0) ||
+		 (url_service->host_pattern_match[0] != '\0') ) {
 
 		/* search Host field in HTTP request header */
 		host = http_search_header_field(request, "Host");
@@ -587,14 +659,28 @@ match_cldata(struct l7vs_service *srv, struct l7vs_conn *conn,
 				if (*(next_line - 1) == '\r') {
 					next_line--;
 				}
-				backup_char = *next_line;
-				*next_line = '\0';
-				/* check keyword (HOST) */
-				ret = fnmatch(pattern, host, 0);
-				*next_line = backup_char;
-				if ( ret != 0 ) {
-					return_value = 1;
-					goto match_cldata_out;
+				/* check keyword (HOST/--host-pattern-match) */
+				if (url_service->host_pattern_match[0] != '\0') {
+					packet_data = std::string(host, next_line - host);
+#ifdef USE_BOOST_XPRESSIVE
+					if ( !boost::xpressive::regex_search(packet_data, url_service->host_regex) ) {
+#else
+					if ( !boost::regex_search(packet_data, url_service->host_regex) ) {
+#endif
+						return_value = -1;
+						goto match_cldata_out;
+					}
+				}
+				/* check keyword (HOST/--pattern-match) */
+				if (url_service->pattern_match[0] != '\0' && ret != 0) {
+					backup_char = *next_line;
+					*next_line = '\0';
+					ret = fnmatch(pattern, host, 0);
+					*next_line = backup_char;
+					if ( ret != 0 ) {
+						return_value = 1;
+						goto match_cldata_out;
+					}
 				}
 			}
 			next_line = NULL;
@@ -930,7 +1016,17 @@ service_arg(struct l7vs_service_arg_multi *srv_arg_mt, handle_t srv_handle)
 	srv_arg_mt->srv_arg.reschedule = url_service->reschedule;
 
 	/* create long argument (l7vsadm option -L/-l) */
-	snprintf(url_argument, SERVICE_ARG_MAXSIZE, "--pattern-match %s", url_service->pattern_match);
+	if ( strnlen(url_service->pattern_match, PATTERN_MATCH_MAXSIZE) ) {
+		snprintf(url_argument, SERVICE_ARG_MAXSIZE, "--pattern-match %s", url_service->pattern_match);
+	}
+	if ( strnlen(url_service->uri_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE) ) {
+		snprintf(url_argument + strlen(url_argument), SERVICE_ARG_MAXSIZE - strlen(url_argument),
+		    " --uri-pattern-match %s", url_service->uri_pattern_match);
+	}
+	if ( strnlen(url_service->host_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE) ) {
+		snprintf(url_argument + strlen(url_argument), SERVICE_ARG_MAXSIZE - strlen(url_argument),
+		    " --host-pattern-match %s", url_service->host_pattern_match);
+	}
 	strncpy(srv_arg_mt->srv_arg.protomod_key_string, url_argument, 256);
 
 	/* create verbose argument (l7vsadm option -V/-v) */
@@ -941,6 +1037,8 @@ service_arg(struct l7vs_service_arg_multi *srv_arg_mt, handle_t srv_handle)
 	strncpy(srv_arg_mt->srv_arg.protomod_opt_string, url_argument, 512);
 
 	strncpy(c_sarg.pattern_match, url_service->pattern_match, PATTERN_MATCH_MAXSIZE);
+	strncpy(c_sarg.uri_pattern_match, url_service->uri_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE);
+	strncpy(c_sarg.host_pattern_match, url_service->host_pattern_match, REGEX_PATTERN_MATCH_MAXSIZE);
 	c_sarg.reschedule = url_service->reschedule;
 
 	memcpy(srv_arg_mt->protomod_arg, &c_sarg, sizeof(struct l7vs_url_service_arg));
@@ -981,14 +1079,21 @@ parse(void *url_arg, int argc, char *argv[])
 {
 	struct l7vs_url_service_arg *url_service_arg;
 	static struct option opt[] = {
-		{"pattern-match",   required_argument, NULL, 'P'},
-		{"forwarded-for", no_argument,       NULL, 'F'},
-		{NULL,              0,                 NULL, 0  }
+		{"pattern-match",      required_argument, NULL, 'P'},
+		{"uri-pattern-match",  required_argument, NULL, 'U'},
+		{"host-pattern-match", required_argument, NULL, 'H'},
+		{"forwarded-for",      no_argument,       NULL, 'F'},
+		{NULL,                 0,                 NULL, 0  }
 	};
 	int c;
 	int pattern_match_flag = 0;
+	int uri_pattern_match_flag = 0;
+	int host_pattern_match_flag = 0;
 	int forwarded_for_flag = 0;
 	int return_value = 0;
+#ifndef USE_BOOST_XPRESSIVE
+    boost::regex regex;
+#endif
 
 	/*-------- DEBUG LOG --------*/
 	if (url_protomod.get_log_level != NULL &&
@@ -1030,7 +1135,7 @@ parse(void *url_arg, int argc, char *argv[])
 	optind = 0;
 
 	/* check all argument */
-	while ((c = getopt_long(argc, argv, "P:F", opt, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "P:U:H:F", opt, NULL)) != -1) {
 		switch (c) {
 		/* --pattern-match / -P */
 		case 'P':
@@ -1052,6 +1157,46 @@ parse(void *url_arg, int argc, char *argv[])
 			pattern_match_flag++;
 			break;
 
+		/* --uri-pattern-match / -U */
+		case 'U':
+			/* check maximum length */
+			if (strnlen(optarg, REGEX_PATTERN_MATCH_MAXSIZE) >= REGEX_PATTERN_MATCH_MAXSIZE) {
+				PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,286,
+				    "-U/--uri-pattern-match option value '%s' too long", optarg);
+				return_value = -1;
+				goto parse_out;
+			}
+			/* check minimum length */
+			if (strnlen(optarg, REGEX_PATTERN_MATCH_MAXSIZE) <= 0 ) {
+				PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,287,
+				    "-U/--uri-pattern-match option value '%s' invalid", optarg);
+				return_value = -1;
+				goto parse_out;
+			}
+			strncpy(url_service_arg->uri_pattern_match, optarg, REGEX_PATTERN_MATCH_MAXSIZE);
+			uri_pattern_match_flag++;
+			break;
+
+		/* --host-pattern-match / -H */
+		case 'H':
+			/* check maximum length */
+			if (strnlen(optarg, REGEX_PATTERN_MATCH_MAXSIZE) >= REGEX_PATTERN_MATCH_MAXSIZE) {
+				PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,288,
+				    "-H/--host-pattern-match option value '%s' too long", optarg);
+				return_value = -1;
+				goto parse_out;
+			}
+			/* check minimum length */
+			if (strnlen(optarg, REGEX_PATTERN_MATCH_MAXSIZE) <= 0 ) {
+				PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,289,
+				    "-H/--host-pattern-match option value '%s' invalid", optarg);
+				return_value = -1;
+				goto parse_out;
+			}
+			strncpy(url_service_arg->host_pattern_match, optarg, REGEX_PATTERN_MATCH_MAXSIZE);
+			host_pattern_match_flag++;
+			break;
+
 		/* --forwarded-for / -F */
 		case 'F':
 			/* x-forwarded-for on */
@@ -1076,9 +1221,9 @@ parse(void *url_arg, int argc, char *argv[])
 	url_service_arg->reschedule = 0;
 
 	/* no pattern-match value */
-	if (pattern_match_flag == 0) {
+	if (pattern_match_flag == 0 && uri_pattern_match_flag == 0 && host_pattern_match_flag ==0) {
 		PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,148,
-		    "You have to set '--pattern-match' option.");
+		    "You have to set any of '--pattern-match', '--uri-pattern-match' or '--host-pattern-match' option.");
 		return_value = -1;
 		goto parse_out;
 	}
@@ -1089,6 +1234,49 @@ parse(void *url_arg, int argc, char *argv[])
 		return_value = -1;
 		goto parse_out;
 	}
+	if (uri_pattern_match_flag > 1) {
+		PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,290,
+		    "Cannot set multiple option '--uri-pattern-match/-U'.");
+		return_value = -1;
+		goto parse_out;
+	}
+	if (host_pattern_match_flag > 1) {
+		PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,291,
+		    "Cannot set multiple option '--host-pattern-match/-H'.");
+		return_value = -1;
+		goto parse_out;
+	}
+	/* compile regex */
+	if (uri_pattern_match_flag) {
+	    try {
+#ifdef USE_BOOST_XPRESSIVE
+	    	boost::xpressive::sregex::compile(url_service_arg->uri_pattern_match);
+#else
+	    	regex.assign(url_service_arg->uri_pattern_match);
+#endif
+	    }
+	    catch (...) {
+			PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,292,
+		  	  "Regex pattern error of option '--uri-pattern-match/-U'.");
+			return_value = -1;
+			goto parse_out;
+	    }
+	}
+	if (host_pattern_match_flag) {
+	    try {
+#ifdef USE_BOOST_XPRESSIVE
+	    	boost::xpressive::sregex::compile(url_service_arg->host_pattern_match);
+#else
+	    	regex.assign(url_service_arg->host_pattern_match);
+#endif
+	    }
+	    catch (...) {
+			PUT_LOG_ERROR(url_protomod, LOG_CAT_L7VSD_PROTOCOL,293,
+		  	  "Regex pattern error of option '--host-pattern-match/-H'.");
+			return_value = -1;
+			goto parse_out;
+	    }
+	}
 
 parse_out:
 	/*-------- DEBUG LOG --------*/
@@ -1276,8 +1464,8 @@ static void l7vs_url_service_c_str(char* buf, struct l7vs_url_service* url) {
 		snprintf(buf, DEBUG_STR_LEN, "NULL");
 	}
 	else {
-		snprintf(buf, DEBUG_STR_LEN, "service_handle=%d, pattern_match=\"%s\", "
-		    "forwarded_for=%d, reschedule=%d", url->service_handle, url->pattern_match,
+		snprintf(buf, DEBUG_STR_LEN, "service_handle=%d, pattern_match=\"%s\", uri_pattern_match=\"%s\", host_pattern_match=\"%s\", "
+		    "forwarded_for=%d, reschedule=%d", url->service_handle, url->pattern_match, url->uri_pattern_match, url->host_pattern_match,
 		    url->forwarded_for, url->reschedule);
 	}
 }
@@ -1292,7 +1480,7 @@ void l7vs_url_service_arg_c_str(char* buf, struct l7vs_url_service_arg* url_arg)
 		snprintf(buf, DEBUG_STR_LEN, "NULL");
 	}
 	else {
-		snprintf(buf, DEBUG_STR_LEN, "pattern_match=\"%s\", forwarded_for=%d, reschedule=%d",
-		    url_arg->pattern_match, url_arg->forwarded_for, url_arg->reschedule);
+		snprintf(buf, DEBUG_STR_LEN, "pattern_match=\"%s\", uri_pattern_match=\"%s\", host_pattern_match=\"%s\", forwarded_for=%d, reschedule=%d",
+		    url_arg->pattern_match, url_arg->uri_pattern_match, url_arg->host_pattern_match, url_arg->forwarded_for, url_arg->reschedule);
 	}
 }
