@@ -250,6 +250,7 @@ l7vs_lsock_get(struct sockaddr_in *sin, uint8_t proto, int backlog)
         }
 
         lsock->refcnt = 1;
+        lsock->fast_schedule = 1;
 
         lsock->iom->callback = l7vs_lsock_callback;
         lsock->iom->data = lsock;
@@ -624,7 +625,10 @@ l7vs_lsock_add_service(struct l7vs_lsock *lsock,
 		"lsock: %s,srv : %p",
 		lsock_str,srv);
 	}
-        lsock->srv_list = g_list_append(lsock->srv_list, srv);
+	lsock->srv_list = g_list_append(lsock->srv_list, srv);
+	if (lsock->fast_schedule) {
+		lsock->fast_schedule = srv->pm->fast_schedule;
+	}
 	if(logger_get_log_level(LOG_CAT_L7VSD_VIRTUAL_SERVICE) == LOG_LV_DEBUG){
                	char lsock_str[DEBUG_STR_LEN] = {0};
                	l7vs_lsock_c_str(lsock_str, lsock);
@@ -646,6 +650,7 @@ void
 l7vs_lsock_remove_service(struct l7vs_lsock *lsock,
                           struct l7vs_service *srv)
 {
+	GList* slist = NULL;
 	if(logger_get_log_level(LOG_CAT_L7VSD_VIRTUAL_SERVICE) == LOG_LV_DEBUG){
                	char lsock_str[DEBUG_STR_LEN] = {0};
                	l7vs_lsock_c_str(lsock_str, lsock);
@@ -656,6 +661,14 @@ l7vs_lsock_remove_service(struct l7vs_lsock *lsock,
 		lsock_str,srv);
 	}
         lsock->srv_list = g_list_remove(lsock->srv_list, srv);
+	lsock->fast_schedule = 1;
+	for (slist = g_list_first(lsock->srv_list); slist != NULL; slist = slist->next) {
+		struct l7vs_service* s = (struct l7vs_service*) slist->data;
+		if (s != NULL && s->pm != NULL && s->pm->fast_schedule == 0) {
+			lsock->fast_schedule = 0;
+			break;
+		}
+	}
 	if(logger_get_log_level(LOG_CAT_L7VSD_VIRTUAL_SERVICE) == LOG_LV_DEBUG){
                	char lsock_str[DEBUG_STR_LEN] = {0};
                	l7vs_lsock_c_str(lsock_str, lsock);
